"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SecretProvider = exports.GitHubAppTokenProvider = exports.GenericSecretProvider = exports.FileSecretProvider = exports.configureAndListen = void 0;
const fastify_1 = require("fastify");
const FileProvider_1 = require("./providers/FileProvider");
Object.defineProperty(exports, "FileSecretProvider", { enumerable: true, get: function () { return FileProvider_1.FileSecretProvider; } });
const GenericProvider_1 = require("./providers/GenericProvider");
Object.defineProperty(exports, "GenericSecretProvider", { enumerable: true, get: function () { return GenericProvider_1.GenericSecretProvider; } });
const GitHubAppProvider_1 = require("./providers/GitHubAppProvider");
Object.defineProperty(exports, "GitHubAppTokenProvider", { enumerable: true, get: function () { return GitHubAppProvider_1.GitHubAppTokenProvider; } });
const SecretProvider_1 = require("./SecretProvider");
Object.defineProperty(exports, "SecretProvider", { enumerable: true, get: function () { return SecretProvider_1.SecretProvider; } });
const handlers_1 = require("./oidc/handlers");
const DEFAULT_PORT = process.env.PORT ? parseInt(process.env.PORT, 10) : 8080;
const configureAndListen = async (config, port = DEFAULT_PORT) => {
    const fastify = (0, fastify_1.default)({
        logger: true,
    });
    fastify.get('/healthcheck', async () => {
        return { alive: true };
    });
    fastify.post('/exchange', async (req, reply) => {
        var _a;
        req.log.info('Received secret exchange request');
        let token = (_a = req.body) === null || _a === void 0 ? void 0 : _a.token;
        if (!token) {
            token = req.headers['x-oidc-token'];
        }
        if (!token || typeof token !== 'string') {
            return reply.code(400).send('Missing token');
        }
        // let validatedClaims: CircleCIOIDCClaims | null = null;
        // const chosenConfig: OIDCSecretExchangeConfig[0] | null = null;
        let filteredSecretProviders = null;
        for (const configuration of config) {
            switch (configuration.type) {
                case 'circleci': {
                    filteredSecretProviders = await (0, handlers_1.getProvidersForConfig)(req.log, configuration, handlers_1.perPlatformHandlers.circleci, token);
                    break;
                }
                case 'github': {
                    filteredSecretProviders = await (0, handlers_1.getProvidersForConfig)(req.log, configuration, handlers_1.perPlatformHandlers.github, token);
                    break;
                }
                case 'invalid': {
                    filteredSecretProviders = await (0, handlers_1.getProvidersForConfig)(req.log, configuration, handlers_1.perPlatformHandlers.invalid, token);
                    break;
                }
            }
        }
        if (!filteredSecretProviders) {
            req.log.warn('Failed to validing incoming OIDC token, did not match any configured organization');
            return reply.code(401).send();
        }
        const secretProvidersInitialized = filteredSecretProviders.providers.map((provider) => provider());
        // Some secret providers need to load content, and some secrets use the same
        // loaded content.  To reduce work we share loadable content across secret providers
        const contentToLoad = {};
        // Get content we need to load and share
        const loaders = [];
        for (const secretProvider of secretProvidersInitialized) {
            const key = secretProvider.loadableContentKey();
            if (contentToLoad[key])
                continue;
            const promise = secretProvider.loadContent();
            contentToLoad[key] = promise;
            loaders.push(promise);
        }
        // Wait for all content to load
        await Promise.all(loaders);
        const secretsToSend = {};
        await Promise.all(secretProvidersInitialized.map(async (provider) => {
            const key = provider.loadableContentKey();
            const providedSecrets = await provider.provideSecrets(await contentToLoad[key]);
            for (const secretKey of Object.keys(providedSecrets)) {
                if (secretsToSend[secretKey]) {
                    throw new Error(`Two secret providers provided the same secret key: "${secretKey}"`);
                }
                secretsToSend[secretKey] = providedSecrets[secretKey];
            }
        }));
        req.log.info(`Respond to a secrets exchange request from "${filteredSecretProviders.subject}" with ${JSON.stringify(Object.keys(secretsToSend))}`);
        if (req.query.format === 'shell' || req.query.format === 'powershell') {
            for (const secretKey of Object.keys(secretsToSend)) {
                if (!/^[A-Za-z][A-Za-z0-9_]+$/i.test(secretKey)) {
                    req.log.error(`Shell format was requested but the "${secretKey}" key is not compatible with shell variable naming`);
                    return reply.status(422).send('');
                }
            }
            if (req.query.format === 'shell') {
                return Object.keys(secretsToSend)
                    .map((secretKey) => `export ${secretKey}=${JSON.stringify(secretsToSend[secretKey])}\n`)
                    .join('');
            }
            if (req.query.format === 'powershell') {
                return Object.keys(secretsToSend)
                    .map((secretKey) => `$env:${secretKey} = ${JSON.stringify(secretsToSend[secretKey])}\n`)
                    .join('');
            }
        }
        return secretsToSend;
    });
    await fastify.listen({
        port,
        host: '0.0.0.0',
    });
};
exports.configureAndListen = configureAndListen;
//# sourceMappingURL=index.js.map